#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// 
    /// </summary>
    [Description("")]
    public class VolumeSpikeMA : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int period = 5; // Default setting for Period
			private bool useMA = false;
		private double multiple = 2.0;
        // User defined variables (add any user defined variables below)
        #endregion

		
		/*
		http://www.traderslaboratory.com/forums/f56/high-volume-spike-reversal-indicator-development-4515.html
		For a "buy signal", I'm looking for price on the bar after the High Volume Spike to close at or above the close of the High Volume Spike bar. As well, I am looking for a close on the bar after the High Volume Spike that is higher or equal to the open, and for the bar to close off of the lows. I basically ensure closing off the lows by this function (Close-Open)<(High-Low).
		*/
        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            //Add(new Plot(Color.FromKnownColor(KnownColor.Orange), PlotStyle.Bar, "Spike"));
			//Add(new Line(Color.DarkGray, 0, "Zero line"));

            CalculateOnBarClose	= true;
            Overlay				= true;
            PriceTypeSupported	= false;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			
//HIGH VOLUME SPIKE LONG SIGNAL
			if (CurrentBar > period)
			{
				// volume condition, with MA alternative
				if ((!useMA && Volume[1]>MAX(Volume,period)[2]) || (useMA && Volume[1] > multiple * SMA(Volume, period )[2]))
				{
					// Long Condition
					if ( (Low[0]<=Low[2]) &&
						(Close[0]>=Close[1]) && (Close[0]>=Open[0]) &&
						(Close[0] > Low[0])) 
					{
						//Signal 1
						if (Low[0]<=Low[1]) BackColor = Color.Green;
						// signal 2
						else if (Low[0]>=Low[1]) BackColor = Color.Lime;
					}
					// short condition
					else if((High[0]>=High[2]) && 
						(Close[0]<=Close[1]) && (Close[0]<=Open[0]) &&
						((Close[0])<(High[0])) )	
					{
						//signal 1
						if (High[0]>=High[1]) BackColor = Color.Crimson;					
						//  signal 2
						else if (High[0]<=High[1])	BackColor = Color.Red;		
							
					}
				}	
			
			
        	}
		}

        #region Properties
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Spike
        {
            get { return Values[0]; }
        }

        [Description("")]
        [Category("Parameters")]
        public int Period
        {
            get { return period; }
            set { period = Math.Max(1, value); }
        }
        [Description("")]
        [Category("Parameters")]
        public bool UseMA 
        {
            get { return useMA; }
            set { useMA = value; }
        }
        [Description("")]
        [Category("Parameters")]
        public double Multiple
        {
            get { return multiple; }
            set { multiple = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private VolumeSpikeMA[] cacheVolumeSpikeMA = null;

        private static VolumeSpikeMA checkVolumeSpikeMA = new VolumeSpikeMA();

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public VolumeSpikeMA VolumeSpikeMA(double multiple, int period, bool useMA)
        {
            return VolumeSpikeMA(Input, multiple, period, useMA);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public VolumeSpikeMA VolumeSpikeMA(Data.IDataSeries input, double multiple, int period, bool useMA)
        {
            checkVolumeSpikeMA.Multiple = multiple;
            multiple = checkVolumeSpikeMA.Multiple;
            checkVolumeSpikeMA.Period = period;
            period = checkVolumeSpikeMA.Period;
            checkVolumeSpikeMA.UseMA = useMA;
            useMA = checkVolumeSpikeMA.UseMA;

            if (cacheVolumeSpikeMA != null)
                for (int idx = 0; idx < cacheVolumeSpikeMA.Length; idx++)
                    if (Math.Abs(cacheVolumeSpikeMA[idx].Multiple - multiple) <= double.Epsilon && cacheVolumeSpikeMA[idx].Period == period && cacheVolumeSpikeMA[idx].UseMA == useMA && cacheVolumeSpikeMA[idx].EqualsInput(input))
                        return cacheVolumeSpikeMA[idx];

            VolumeSpikeMA indicator = new VolumeSpikeMA();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Multiple = multiple;
            indicator.Period = period;
            indicator.UseMA = useMA;
            indicator.SetUp();

            VolumeSpikeMA[] tmp = new VolumeSpikeMA[cacheVolumeSpikeMA == null ? 1 : cacheVolumeSpikeMA.Length + 1];
            if (cacheVolumeSpikeMA != null)
                cacheVolumeSpikeMA.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheVolumeSpikeMA = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.VolumeSpikeMA VolumeSpikeMA(double multiple, int period, bool useMA)
        {
            return _indicator.VolumeSpikeMA(Input, multiple, period, useMA);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Indicator.VolumeSpikeMA VolumeSpikeMA(Data.IDataSeries input, double multiple, int period, bool useMA)
        {
            return _indicator.VolumeSpikeMA(input, multiple, period, useMA);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.VolumeSpikeMA VolumeSpikeMA(double multiple, int period, bool useMA)
        {
            return _indicator.VolumeSpikeMA(Input, multiple, period, useMA);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Indicator.VolumeSpikeMA VolumeSpikeMA(Data.IDataSeries input, double multiple, int period, bool useMA)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.VolumeSpikeMA(input, multiple, period, useMA);
        }

    }
}
#endregion
